@@ -7,6 +7,8 @@ Optimal binning information.
 
 import numpy as np
 
+from sklearn.base import BaseEstimator
+
 from .options import continuous_optimal_binning_default_options
 from .options import multiclass_optimal_binning_default_options
 from .options import optimal_binning_default_options
@@ -21,7 +23,7 @@ except ImportError:
 
 def print_header():
     header = (
-        "optbinning (Version 0.5.0)\n"
+        "optbinning (Version 0.6.1)\n"
         "Copyright (c) 2019-2020 Guillermo Navas-Palencia, Apache License 2.0"
         "\n")
 
@@ -39,6 +41,8 @@ def print_optional_parameters(dict_default_options, dict_user_options):
             user_value = "no"
         elif isinstance(user_value, (list, np.ndarray, dict)):
             user_value = "yes"
+        elif isinstance(user_value, BaseEstimator):
+            user_value = "yes"
 
         str_options += option_format.format(key, str(user_value), user_flag)
     str_options += "  End options\n"
