@@ -55,6 +55,8 @@ COLORS_RGB = [
 
 
 def bin_str_format(bins, show_digits):
+    # Auto
+    show_digits = 2 if show_digits is None else show_digits
     return ["[{0:.{2}f}, {1:.{2}f})".format(bins[i], bins[i+1], show_digits)
             for i in range(len(bins)-1)]
 
@@ -63,7 +65,7 @@ def bin_categorical(splits_categorical, categories, cat_others, user_splits):
     splits = np.ceil(splits_categorical).astype(np.int)
     n_categories = len(categories)
 
-    if user_splits:
+    if user_splits is not None:
         indices = np.digitize(np.arange(n_categories), splits, right=True)
         n_bins = len(splits)
     else:
@@ -102,6 +104,19 @@ def target_info(y, cl=0):
         return n_nonevent, n_event
 
 
+def target_info_samples(y, sw, cl=0):
+    if not len(y):
+        return 0, 0
+    elif not len(sw):
+        return target_info(y, cl)
+    else:
+        y0 = (y == cl)
+        n_nonevent = np.sum(sw[y0])
+        n_event = np.sum(sw[~y0])
+
+        return n_nonevent, n_event
+
+
 def bin_info(solution, n_nonevent, n_event, n_nonevent_missing,
              n_event_missing, n_nonevent_special, n_event_special,
              n_nonevent_cat_others, n_event_cat_others, cat_others):
@@ -409,7 +424,8 @@ class BinningTable:
 
         return df
 
-    def plot(self, metric="woe", savefig=None):
+    def plot(self, metric="woe", add_special=True, add_missing=True,
+             savefig=None):
         """Plot the binning table.
 
         Visualize the non-event and event count, and the Weight of Evidence or
@@ -421,6 +437,12 @@ class BinningTable:
             Supported metrics are "woe" to show the Weight of Evidence (WoE)
             measure and "event_rate" to show the event rate.
 
+        add_special : bool (default=True)
+            Whether to add the special codes bin.
+
+        add_missing : bool (default=True)
+            Whether to add the special values bin.
+
         savefig : str or None (default=None)
             Path to save the plot figure.
         """
@@ -430,17 +452,38 @@ class BinningTable:
             raise ValueError('Invalid value for metric. Allowed string '
                              'values are "event_rate" and "woe".')
 
+        if not isinstance(add_special, bool):
+            raise TypeError("add_special must be a boolean; got {}."
+                            .format(add_special))
+
+        if not isinstance(add_missing, bool):
+            raise TypeError("add_missing must be a boolean; got {}."
+                            .format(add_missing))
+
         n_bins = len(self._n_records)
         n_metric = n_bins - 2
 
         if len(self.cat_others):
             n_metric -= 1
 
+        _n_event = list(self.n_event)
+        _n_nonevent = list(self.n_nonevent)
+
+        if not add_special:
+            _n_event.pop(-2)
+            _n_nonevent.pop(-2)
+            n_bins -= 1
+
+        if not add_missing:
+            _n_event.pop(-1)
+            _n_nonevent.pop(-1)
+            n_bins -= 1
+
         fig, ax1 = plt.subplots()
 
-        p2 = ax1.bar(range(n_bins), self.n_event, color="tab:red")
-        p1 = ax1.bar(range(n_bins), self.n_nonevent, color="tab:blue",
-                     bottom=self.n_event)
+        p2 = ax1.bar(range(n_bins), _n_event, color="tab:red")
+        p1 = ax1.bar(range(n_bins), _n_nonevent, color="tab:blue",
+                     bottom=_n_event)
 
         handles = [p1[0], p2[0]]
         labels = ['Non-event', 'Event']
@@ -460,39 +503,53 @@ class BinningTable:
         ax2.plot(range(n_metric), metric_values[:n_metric], linestyle="solid",
                  marker="o", color="black")
 
+        # Positions special and missing bars
+        if add_special:
+            pos_special = n_metric
+            if add_missing:
+                pos_missing = n_metric + 1
+        elif add_missing:
+            pos_missing = n_metric
+
         # Add points for others (optional), special and missing bin
         if len(self.cat_others):
             pos_others = n_metric
-            pos_special = n_metric + 1
-            pos_missing = n_metric + 2
+            pos_special += 1
+            pos_missing += 1
 
             p1[pos_others].set_alpha(0.5)
             p2[pos_others].set_alpha(0.5)
 
             ax2.plot(pos_others, metric_values[pos_others], marker="o",
                      color="black")
-        else:
-            pos_special = n_metric
-            pos_missing = n_metric + 1
 
-        p1[pos_special].set_hatch("/")
-        p2[pos_special].set_hatch("/")
-        p1[pos_missing].set_hatch("\\")
-        p2[pos_missing].set_hatch("\\")
+        if add_special:
+            p1[pos_special].set_hatch("/")
+            p2[pos_special].set_hatch("/")
+            handle_special = mpatches.Patch(hatch="/", alpha=0.1)
+            label_special = "Bin special"
 
-        handle_special = mpatches.Patch(hatch="/", alpha=0.1)
-        label_special = "Bin special"
+            ax2.plot(pos_special, metric_values[pos_special], marker="o",
+                     color="black")
 
-        handle_missing = mpatches.Patch(hatch="\\", alpha=0.1)
-        label_missing = "Bin missing"
+        if add_missing:
+            p1[pos_missing].set_hatch("\\")
+            p2[pos_missing].set_hatch("\\")
+            handle_missing = mpatches.Patch(hatch="\\", alpha=0.1)
+            label_missing = "Bin missing"
 
-        handles.extend([handle_special, handle_missing])
-        labels.extend([label_special, label_missing])
+            ax2.plot(pos_missing, metric_values[pos_missing], marker="o",
+                     color="black")
 
-        ax2.plot(pos_special, metric_values[pos_special], marker="o",
-                 color="black")
-        ax2.plot(pos_missing, metric_values[pos_missing], marker="o",
-                 color="black")
+        if add_special and add_missing:
+            handles.extend([handle_special, handle_missing])
+            labels.extend([label_special, label_missing])
+        elif add_special:
+            handles.extend([handle_special])
+            labels.extend([label_special])
+        elif add_missing:
+            handles.extend([handle_missing])
+            labels.extend([label_missing])
 
         ax2.set_ylabel(metric_label, fontsize=13)
         ax2.xaxis.set_major_locator(mtick.MultipleLocator(1))
@@ -565,7 +622,10 @@ class BinningTable:
         n_nev = self.n_nonevent[:n_metric]
         n_ev = self.n_event[:n_metric]
 
-        chi2, cramer_v = chi2_cramer_v(n_nev, n_ev)
+        if len(n_nev) >= 2:
+            chi2, cramer_v = chi2_cramer_v(n_nev, n_ev)
+        else:
+            cramer_v = 0
 
         t_statistics = []
         p_values = []
@@ -798,13 +858,19 @@ class MulticlassBinningTable:
 
         return df
 
-    def plot(self, savefig=None):
+    def plot(self, add_special=True, add_missing=True, savefig=None):
         """Plot the binning table.
 
         Visualize event count and event rate values for each class.
 
         Parameters
         ----------
+        add_special : bool (default=True)
+            Whether to add the special codes bin.
+
+        add_missing : bool (default=True)
+            Whether to add the special values bin.
+
         savefig : str or None (default=None)
             Path to save the plot figure.
         """
@@ -819,13 +885,29 @@ class MulticlassBinningTable:
         colors = COLORS_RGB[:n_classes]
         colors = [tuple(c / 255. for c in color) for color in colors]
 
-        p = []
-        cum_size = np.zeros(len(self.n_event))
+        if not add_special:
+            n_bins -= 1
 
+        if not add_missing:
+            n_bins -= 1
+
+        _n_event = []
+        for i in range(n_classes):
+            _n_event_c = list(self.n_event[:, i])
+            if not add_special:
+                _n_event_c.pop(-2)
+            if not add_missing:
+                _n_event_c.pop(-1)
+            _n_event.append(np.array(_n_event_c))
+
+        _n_event = np.array(_n_event)
+
+        p = []
+        cum_size = np.zeros(n_bins)
         for i, cl in enumerate(self.classes):
-            p.append(ax1.bar(range(n_bins), self.n_event[:, i],
+            p.append(ax1.bar(range(n_bins), _n_event[i],
                              color=colors[i], bottom=cum_size))
-            cum_size += self.n_event[:, i]
+            cum_size += _n_event[i]
 
         handles = [_p[0] for _p in p]
         labels = list(self.classes)
@@ -844,27 +926,44 @@ class MulticlassBinningTable:
                      markerfacecolor=colors[i], markeredgewidth=0.5)
 
         # Add points for special and missing bin
-        pos_special = n_metric
-        pos_missing = n_metric + 1
+        if add_special:
+            pos_special = n_metric
+            if add_missing:
+                pos_missing = n_metric + 1
+        elif add_missing:
+            pos_missing = n_metric
 
-        for _p in p:
-            _p[pos_special].set_hatch("/")
-            _p[pos_missing].set_hatch("\\")
+        if add_special:
+            for _p in p:
+                _p[pos_special].set_hatch("/")
 
-        handle_special = mpatches.Patch(hatch="/", alpha=0.1)
-        label_special = "Bin special"
+            handle_special = mpatches.Patch(hatch="/", alpha=0.1)
+            label_special = "Bin special"
 
-        handle_missing = mpatches.Patch(hatch="\\", alpha=0.1)
-        label_missing = "Bin missing"
+            for i, cl in enumerate(self.classes):
+                ax2.plot(pos_special, metric_values[pos_special, i],
+                         marker="o", color=colors[i])
 
-        handles.extend([handle_special, handle_missing])
-        labels.extend([label_special, label_missing])
+        if add_missing:
+            for _p in p:
+                _p[pos_missing].set_hatch("\\")
 
-        for i, cl in enumerate(self.classes):
-            ax2.plot(pos_special, metric_values[pos_special, i], marker="o",
-                     color=colors[i])
-            ax2.plot(pos_missing, metric_values[pos_missing, i], marker="o",
-                     color=colors[i])
+            handle_missing = mpatches.Patch(hatch="\\", alpha=0.1)
+            label_missing = "Bin missing"
+
+            for i, cl in enumerate(self.classes):
+                ax2.plot(pos_missing, metric_values[pos_missing, i],
+                         marker="o", color=colors[i])
+
+        if add_special and add_missing:
+            handles.extend([handle_special, handle_missing])
+            labels.extend([label_special, label_missing])
+        elif add_special:
+            handles.extend([handle_special])
+            labels.extend([label_special])
+        elif add_missing:
+            handles.extend([handle_missing])
+            labels.extend([label_missing])
 
         ax2.set_ylabel(metric_label, fontsize=13)
         ax2.xaxis.set_major_locator(mtick.MultipleLocator(1))
@@ -908,7 +1007,10 @@ class MulticlassBinningTable:
         n_metric = n_bins - 2
 
         n_ev = self.n_event[:n_metric, :]
-        chi2, cramer_v = chi2_cramer_v_multi(n_ev)
+        if len(n_ev) >= 2:
+            chi2, cramer_v = chi2_cramer_v_multi(n_ev)
+        else:
+            cramer_v = 0
 
         t_statistics = []
         p_values = []
@@ -1074,14 +1176,20 @@ class ContinuousBinningTable:
         """
         _check_build_parameters(show_digits, add_totals)
 
-        t_n_records = self.n_records.sum()
-        t_sum = self.sums.sum()
+        t_n_records = np.nansum(self.n_records)
+        t_sum = np.nansum(self.sums)
         t_mean = t_sum / t_n_records
         p_records = self.n_records / t_n_records
 
         mask = (self.n_records > 0)
         self._mean = np.zeros(len(self.n_records))
         self._mean[mask] = self.sums[mask] / self.n_records[mask]
+        if self.n_records[-1] > 0:
+            self._mean[-1] = 0
+
+        woe = self._mean - t_mean
+        iv = np.absolute(woe) * p_records
+        t_iv = iv.sum()
 
         if self.dtype == "numerical":
             bins = np.concatenate([[-np.inf], self.splits, [np.inf]])
@@ -1100,7 +1208,9 @@ class ContinuousBinningTable:
             "Mean": self._mean,
             "Min": self.min_target,
             "Max": self.max_target,
-            "Zeros count": self.n_zeros
+            "Zeros count": self.n_zeros,
+            "WoE": woe,
+            "IV": iv,
             })
 
         if add_totals:
@@ -1108,34 +1218,58 @@ class ContinuousBinningTable:
             t_max = np.max(self.max_target)
             t_n_zeros = self.n_zeros.sum()
             totals = ["", t_n_records, 1, t_sum, t_mean, t_min, t_max,
-                      t_n_zeros]
+                      t_n_zeros, "", t_iv]
             df.loc["Totals"] = totals
 
         self._is_built = True
 
         return df
 
-    def plot(self, savefig=None):
+    def plot(self, add_special=True, add_missing=True, savefig=None):
         """Plot the binning table.
 
         Visualize records count and mean values.
 
         Parameters
         ----------
+        add_special : bool (default=True)
+            Whether to add the special codes bin.
+
+        add_missing : bool (default=True)
+            Whether to add the special values bin.
+
         savefig : str or None (default=None)
             Path to save the plot figure.
         """
         _check_is_built(self)
 
+        if not isinstance(add_special, bool):
+            raise TypeError("add_special must be a boolean; got {}."
+                            .format(add_special))
+
+        if not isinstance(add_missing, bool):
+            raise TypeError("add_missing must be a boolean; got {}."
+                            .format(add_missing))
+
         n_bins = len(self.n_records)
         n_metric = n_bins - 2
 
         if len(self.cat_others):
             n_metric -= 1
 
+        _n_records = list(self.n_records)
+
+        if not add_special:
+            _n_records.pop(-2)
+            n_bins -= 1
+
+        if not add_missing:
+            _n_records.pop(-1)
+            n_bins -= 1
+
         fig, ax1 = plt.subplots()
 
-        p1 = ax1.bar(range(n_bins), self.n_records, color="tab:blue")
+        p1 = ax1.bar(range(n_bins), _n_records, color="tab:blue")
 
         handles = [p1[0]]
         labels = ['Count']
@@ -1151,36 +1285,53 @@ class ContinuousBinningTable:
         ax2.plot(range(n_metric), metric_values[:n_metric], linestyle="solid",
                  marker="o", color="black")
 
+        # Positions special and missing bars
+        pos_special = 0
+        pos_missing = 0
+
+        if add_special:
+            pos_special = n_metric
+            if add_missing:
+                pos_missing = n_metric + 1
+        elif add_missing:
+            pos_missing = n_metric
+
         # Add points for others (optional), special and missing bin
         if len(self.cat_others):
             pos_others = n_metric
-            pos_special = n_metric + 1
-            pos_missing = n_metric + 2
+            pos_special += 1
+            pos_missing += 1
 
             p1[pos_others].set_alpha(0.5)
 
             ax2.plot(pos_others, metric_values[pos_others], marker="o",
                      color="black")
-        else:
-            pos_special = n_metric
-            pos_missing = n_metric + 1
 
-        p1[pos_special].set_hatch("/")
-        p1[pos_missing].set_hatch("\\")
+        if add_special:
+            p1[pos_special].set_hatch("/")
+            handle_special = mpatches.Patch(hatch="/", alpha=0.1)
+            label_special = "Bin special"
 
-        handle_special = mpatches.Patch(hatch="/", alpha=0.1)
-        label_special = "Bin special"
+            ax2.plot(pos_special, metric_values[pos_special], marker="o",
+                     color="black")
 
-        handle_missing = mpatches.Patch(hatch="\\", alpha=0.1)
-        label_missing = "Bin missing"
+        if add_missing:
+            p1[pos_missing].set_hatch("\\")
+            handle_missing = mpatches.Patch(hatch="\\", alpha=0.1)
+            label_missing = "Bin missing"
 
-        handles.extend([handle_special, handle_missing])
-        labels.extend([label_special, label_missing])
+            ax2.plot(pos_missing, metric_values[pos_missing], marker="o",
+                     color="black")
 
-        ax2.plot(pos_special, metric_values[pos_special], marker="o",
-                 color="black")
-        ax2.plot(pos_missing, metric_values[pos_missing], marker="o",
-                 color="black")
+        if add_special and add_missing:
+            handles.extend([handle_special, handle_missing])
+            labels.extend([label_special, label_missing])
+        elif add_special:
+            handles.extend([handle_special])
+            labels.extend([label_special])
+        elif add_missing:
+            handles.extend([handle_missing])
+            labels.extend([label_missing])
 
         ax2.set_ylabel(metric_label, fontsize=13)
         ax2.xaxis.set_major_locator(mtick.MultipleLocator(1))
