@@ -5,7 +5,6 @@ Optimal binning algorithm for continuous target.
 # Guillermo Navas-Palencia <g.navas.palencia@gmail.com>
 # Copyright (C) 2019
 
-import logging
 import numbers
 import time
 
@@ -269,6 +268,11 @@ class ContinuousOptimalBinning(OptimalBinning):
     verbose : bool (default=False)
         Enable verbose output.
 
+    **prebinning_kwargs : keyword arguments
+        The pre-binning keywrord arguments.
+
+        .. versionadded:: 0.6.1
+
     Notes
     -----
     The parameter values ``max_n_prebins`` and ``min_prebin_size`` control
@@ -289,7 +293,7 @@ class ContinuousOptimalBinning(OptimalBinning):
                  max_pvalue_policy="consecutive", outlier_detector=None,
                  outlier_params=None, cat_cutoff=None, user_splits=None,
                  user_splits_fixed=None, special_codes=None, split_digits=None,
-                 time_limit=100, verbose=False):
+                 time_limit=100, verbose=False, **prebinning_kwargs):
 
         self.name = name
         self.dtype = dtype
@@ -322,6 +326,7 @@ class ContinuousOptimalBinning(OptimalBinning):
         self.time_limit = time_limit
 
         self.verbose = verbose
+        self.prebinning_kwargs = prebinning_kwargs
 
         # auxiliary
         self._categories = None
@@ -365,12 +370,34 @@ class ContinuousOptimalBinning(OptimalBinning):
         self._time_postprocessing = None
 
         # logger
-        self._logger = Logger()
+        self._class_logger = Logger(__name__)
+        self._logger = self._class_logger.logger
 
         self._is_fitted = False
 
-    def fit_transform(self, x, y, metric_special=0, metric_missing=0,
-                      check_input=False):
+    def fit(self, x, y, check_input=False):
+        """Fit the optimal binning according to the given training data.
+
+        Parameters
+        ----------
+        x : array-like, shape = (n_samples,)
+            Training vector, where n_samples is the number of samples.
+
+        y : array-like, shape = (n_samples,)
+            Target vector relative to x.
+
+        check_input : bool (default=False)
+            Whether to check input arrays.
+
+        Returns
+        -------
+        self : object
+            Fitted optimal binning.
+        """
+        return self._fit(x, y, check_input)
+
+    def fit_transform(self, x, y, metric="mean", metric_special=0,
+                      metric_missing=0, show_digits=2, check_input=False):
         """Fit the optimal binning according to the given training data, then
         transform it.
 
@@ -382,6 +409,12 @@ class ContinuousOptimalBinning(OptimalBinning):
         y : array-like, shape = (n_samples,)
             Target vector relative to x.
 
+        metric : str (default="mean"):
+            The metric used to transform the input vector. Supported metrics
+            are "mean" to choose the mean, "indices" to assign the
+            corresponding indices of the bins and "bins" to assign the
+            corresponding bin interval.
+
         metric_special : float or str (default=0)
             The metric value to transform special codes in the input vector.
             Supported metrics are "empirical" to use the empirical mean, and
@@ -392,6 +425,10 @@ class ContinuousOptimalBinning(OptimalBinning):
             Supported metrics are "empirical" to use the empirical mean, and
             any numerical value.
 
+        show_digits : int, optional (default=2)
+            The number of significant digits of the bin column. Applies when
+            ``metric="bins"``.
+
         check_input : bool (default=False)
             Whether to check input arrays.
 
@@ -401,10 +438,11 @@ class ContinuousOptimalBinning(OptimalBinning):
             Transformed array.
         """
         return self.fit(x, y, check_input).transform(
-            x, metric_special, metric_missing, check_input)
+            x, metric, metric_special, metric_missing, show_digits,
+            check_input)
 
-    def transform(self, x, metric_special=0, metric_missing=0,
-                  check_input=False):
+    def transform(self, x, metric="mean", metric_special=0, metric_missing=0,
+                  show_digits=2, check_input=False):
         """Transform given data to mean using bins from the fitted
         optimal binning.
 
@@ -413,6 +451,12 @@ class ContinuousOptimalBinning(OptimalBinning):
         x : array-like, shape = (n_samples,)
             Training vector, where n_samples is the number of samples.
 
+        metric : str (default="mean"):
+            The metric used to transform the input vector. Supported metrics
+            are "mean" to choose the mean, "indices" to assign the
+            corresponding indices of the bins and "bins" to assign the
+            corresponding bin interval.
+
         metric_special : float or str (default=0)
             The metric value to transform special codes in the input vector.
             Supported metrics are "empirical" to use the empirical mean, and
@@ -423,6 +467,10 @@ class ContinuousOptimalBinning(OptimalBinning):
             Supported metrics are "empirical" to use the empirical mean, and
             any numerical value.
 
+        show_digits : int, optional (default=2)
+            The number of significant digits of the bin column. Applies when
+            ``metric="bins"``.
+
         check_input : bool (default=False)
             Whether to check input arrays.
 
@@ -442,32 +490,33 @@ class ContinuousOptimalBinning(OptimalBinning):
                                            x, self._n_records, self._sums,
                                            self.special_codes,
                                            self._categories, self._cat_others,
-                                           metric_special, metric_missing,
-                                           self.user_splits, check_input)
+                                           metric, metric_special,
+                                           metric_missing, self.user_splits,
+                                           show_digits, check_input)
 
     def _fit(self, x, y, check_input):
         time_init = time.perf_counter()
 
         if self.verbose:
-            logging.info("Optimal binning started.")
-            logging.info("Options: check parameters.")
+            self._logger.info("Optimal binning started.")
+            self._logger.info("Options: check parameters.")
 
         _check_parameters(**self.get_params())
 
         # Pre-processing
         if self.verbose:
-            logging.info("Pre-processing started.")
+            self._logger.info("Pre-processing started.")
 
         self._n_samples = len(x)
 
         if self.verbose:
-            logging.info("Pre-processing: number of samples: {}"
-                         .format(self._n_samples))
+            self._logger.info("Pre-processing: number of samples: {}"
+                              .format(self._n_samples))
 
         time_preprocessing = time.perf_counter()
 
         [x_clean, y_clean, x_missing, y_missing, x_special, y_special,
-         y_others, categories, cat_others] = split_data(
+         y_others, categories, cat_others, _, _, _, _] = split_data(
             self.dtype, x, y, self.special_codes, self.cat_cutoff,
             self.user_splits, check_input, self.outlier_detector,
             self.outlier_params)
@@ -479,40 +528,40 @@ class ContinuousOptimalBinning(OptimalBinning):
             n_missing = len(x_missing)
             n_special = len(x_special)
 
-            logging.info("Pre-processing: number of clean samples: {}"
-                         .format(n_clean))
+            self._logger.info("Pre-processing: number of clean samples: {}"
+                              .format(n_clean))
 
-            logging.info("Pre-processing: number of missing samples: {}"
-                         .format(n_missing))
+            self._logger.info("Pre-processing: number of missing samples: {}"
+                              .format(n_missing))
 
-            logging.info("Pre-processing: number of special samples: {}"
-                         .format(n_special))
+            self._logger.info("Pre-processing: number of special samples: {}"
+                              .format(n_special))
 
             if self.outlier_detector is not None:
                 n_outlier = self._n_samples-(n_clean + n_missing + n_special)
-                logging.info("Pre-processing: number of outlier samples: {}"
-                             .format(n_outlier))
+                self._logger.info("Pre-processing: number of outlier samples: "
+                                  "{}".format(n_outlier))
 
             if self.dtype == "categorical":
                 n_categories = len(categories)
                 n_categories_others = len(cat_others)
                 n_others = len(y_others)
 
-                logging.info("Pre-processing: number of others samples: {}"
-                             .format(n_others))
+                self._logger.info("Pre-processing: number of others samples: "
+                                  "{}".format(n_others))
 
-                logging.info("Pre-processing: number of categories: {}"
-                             .format(n_categories))
+                self._logger.info("Pre-processing: number of categories: {}"
+                                  .format(n_categories))
 
-                logging.info("Pre-processing: number of categories others: {}"
-                             .format(n_categories_others))
+                self._logger.info("Pre-processing: number of categories "
+                                  "others: {}".format(n_categories_others))
 
-            logging.info("Pre-processing terminated. Time: {:.4f}s"
-                         .format(self._time_preprocessing))
+            self._logger.info("Pre-processing terminated. Time: {:.4f}s"
+                              .format(self._time_preprocessing))
 
         # Pre-binning
         if self.verbose:
-            logging.info("Pre-binning started.")
+            self._logger.info("Pre-binning started.")
 
         time_prebinning = time.perf_counter()
 
@@ -520,8 +569,8 @@ class ContinuousOptimalBinning(OptimalBinning):
             n_splits = len(self.user_splits)
 
             if self.verbose:
-                logging.info("Pre-binning: user splits supplied: {}"
-                             .format(n_splits))
+                self._logger.info("Pre-binning: user splits supplied: {}"
+                                  .format(n_splits))
 
             if not n_splits:
                 splits = self.user_splits
@@ -534,11 +583,20 @@ class ContinuousOptimalBinning(OptimalBinning):
                         self.user_splits, ensure_2d=False, dtype=None,
                         force_all_finite=True)
 
-                    user_splits = np.unique(self.user_splits)
+                    if len(set(user_splits)) != len(user_splits):
+                        raise ValueError("User splits are not unique.")
+
+                    sorted_idx = np.argsort(user_splits)
+                    user_splits = user_splits[sorted_idx]
                 else:
                     [categories, user_splits, x_clean, y_clean, y_others,
-                     cat_others] = preprocessing_user_splits_categorical(
-                        self.user_splits, x_clean, y_clean)
+                     cat_others, _, _, sorted_idx
+                     ] = preprocessing_user_splits_categorical(
+                        self.user_splits, x_clean, y_clean, None)
+
+                if self.user_splits_fixed is not None:
+                    self.user_splits_fixed = np.asarray(
+                        self.user_splits_fixed)[sorted_idx]
 
                 [splits, n_records, sums, stds, min_t, max_t,
                  n_zeros] = self._prebinning_refinement(
@@ -557,19 +615,19 @@ class ContinuousOptimalBinning(OptimalBinning):
         self._time_prebinning = time.perf_counter() - time_prebinning
 
         if self.verbose:
-            logging.info("Pre-binning: number of prebins: {}"
-                         .format(self._n_prebins))
+            self._logger.info("Pre-binning: number of prebins: {}"
+                              .format(self._n_prebins))
 
-            logging.info("Pre-binning terminated. Time: {:.4f}s"
-                         .format(self._time_prebinning))
+            self._logger.info("Pre-binning terminated. Time: {:.4f}s"
+                              .format(self._time_prebinning))
 
         # Optimization
         self._fit_optimizer(splits, n_records, sums, stds)
 
         # Post-processing
         if self.verbose:
-            logging.info("Post-processing started.")
-            logging.info("Post-processing: compute binning information.")
+            self._logger.info("Post-processing started.")
+            self._logger.info("Post-processing: compute binning information.")
 
         time_postprocessing = time.perf_counter()
 
@@ -596,25 +654,25 @@ class ContinuousOptimalBinning(OptimalBinning):
         self._time_postprocessing = time.perf_counter() - time_postprocessing
 
         if self.verbose:
-            logging.info("Post-processing terminated. Time: {:.4f}s"
-                         .format(self._time_postprocessing))
+            self._logger.info("Post-processing terminated. Time: {:.4f}s"
+                              .format(self._time_postprocessing))
 
         self._time_total = time.perf_counter() - time_init
 
         if self.verbose:
-            logging.info("Optimal binning terminated. Status: {}. "
-                         "Time: {:.4f}s".format(
-                            self._status, self._time_total))
+            self._logger.info("Optimal binning terminated. Status: {}. "
+                              "Time: {:.4f}s"
+                              .format(self._status, self._time_total))
 
         # Completed successfully
-        self._logger.close()
+        self._class_logger.close()
         self._is_fitted = True
 
         return self
 
     def _fit_optimizer(self, splits, n_records, sums, stds):
         if self.verbose:
-            logging.info("Optimizer started.")
+            self._logger.info("Optimizer started.")
 
         time_init = time.perf_counter()
 
@@ -624,10 +682,10 @@ class ContinuousOptimalBinning(OptimalBinning):
             self._solution = np.zeros(len(splits)).astype(np.bool)
 
             if self.verbose:
-                logging.warning("Optimizer: no bins after pre-binning.")
-                logging.warning("Optimizer: solver not run.")
+                self._logger.warning("Optimizer: no bins after pre-binning.")
+                self._logger.warning("Optimizer: solver not run.")
 
-                logging.info("Optimizer terminated. Time: 0s")
+                self._logger.info("Optimizer terminated. Time: 0s")
             return
 
         if self.min_bin_size is not None:
@@ -661,8 +719,8 @@ class ContinuousOptimalBinning(OptimalBinning):
                             mean, monotonic)
 
                 if self.verbose:
-                    logging.info("Optimizer: classifier predicts {} monotonic "
-                                 "trend.".format(monotonic))
+                    self._logger.info("Optimizer: classifier predicts {} "
+                                      "monotonic trend.".format(monotonic))
             else:
                 monotonic = self.monotonic_trend
 
@@ -671,18 +729,18 @@ class ContinuousOptimalBinning(OptimalBinning):
                     trend_change = peak_valley_trend_change_heuristic(
                         mean, monotonic)
 
-                if self.verbose:
-                    if monotonic is None:
-                        logging.info("Optimizer: monotonic trend not set.")
-                    else:
-                        logging.info("Optimizer: monotonic trend set to {}."
-                                     .format(monotonic))
         else:
-            monotonic = "ascending"
+            monotonic = self.monotonic_trend
+            if monotonic is not None:
+                monotonic = "ascending"
 
-            if self.verbose:
-                logging.info("Optimizer: monotonic trend set to ascending for "
-                             "categorical dtype.")
+        if self.verbose:
+            if monotonic is None:
+                self._logger.info(
+                    "Optimizer: monotonic trend not set.")
+            else:
+                self._logger.info("Optimizer: monotonic trend set to "
+                                  "{}.".format(monotonic))
 
         optimizer = ContinuousBinningCP(monotonic, self.min_n_bins,
                                         self.max_n_bins, min_bin_size,
@@ -693,12 +751,12 @@ class ContinuousOptimalBinning(OptimalBinning):
                                         self.time_limit)
 
         if self.verbose:
-            logging.info("Optimizer: build model...")
+            self._logger.info("Optimizer: build model...")
 
         optimizer.build_model(n_records, sums, stds, trend_change)
 
         if self.verbose:
-            logging.info("Optimizer: solve...")
+            self._logger.info("Optimizer: solve...")
 
         status, solution = optimizer.solve()
 
@@ -715,11 +773,13 @@ class ContinuousOptimalBinning(OptimalBinning):
         self._time_solver = time.perf_counter() - time_init
 
         if self.verbose:
-            logging.info("Optimizer terminated. Time: {:.4f}s"
-                         .format(self._time_solver))
+            self._logger.info("Optimizer terminated. Time: {:.4f}s"
+                              .format(self._time_solver))
 
     def _prebinning_refinement(self, splits_prebinning, x, y, y_missing,
-                               y_special, y_others):
+                               y_special, y_others, sw_clean=None,
+                               sw_missing=None, sw_special=None,
+                               sw_others=None):
         n_splits = len(splits_prebinning)
 
         if not n_splits:
