@@ -5,7 +5,6 @@ Optimal binning algorithm for multiclass target.
 # Guillermo Navas-Palencia <g.navas.palencia@gmail.com>
 # Copyright (C) 2019
 
-import logging
 import numbers
 import time
 
@@ -268,6 +267,11 @@ class MulticlassOptimalBinning(OptimalBinning):
     verbose : bool (default=False)
         Enable verbose output.
 
+    **prebinning_kwargs : keyword arguments
+        The pre-binning keywrord arguments.
+
+        .. versionadded:: 0.6.1
+
     Notes
     -----
     The parameter values ``max_n_prebins`` and ``min_prebin_size`` control
@@ -286,7 +290,7 @@ class MulticlassOptimalBinning(OptimalBinning):
                  max_pvalue_policy="consecutive", outlier_detector=None,
                  outlier_params=None, user_splits=None, user_splits_fixed=None,
                  special_codes=None, split_digits=None, mip_solver="bop",
-                 time_limit=100, verbose=False):
+                 time_limit=100, verbose=False, **prebinning_kwargs):
 
         self.name = name
         self.dtype = "numerical"
@@ -317,14 +321,13 @@ class MulticlassOptimalBinning(OptimalBinning):
         self.time_limit = time_limit
 
         self.verbose = verbose
+        self.prebinning_kwargs = prebinning_kwargs
 
         # auxiliary
         self._n_event = None
         self._n_event_missing = None
         self._n_event_special = None
         self._problem_type = "classification"
-        self._user_splits = user_splits
-        self._user_splits_fixed = user_splits_fixed
 
         # info
         self._binning_table = None
@@ -345,12 +348,34 @@ class MulticlassOptimalBinning(OptimalBinning):
         self._time_postprocessing = None
 
         # logger
-        self._logger = Logger()
+        self._class_logger = Logger(__name__)
+        self._logger = self._class_logger.logger
 
         self._is_fitted = False
 
+    def fit(self, x, y, check_input=False):
+        """Fit the optimal binning according to the given training data.
+
+        Parameters
+        ----------
+        x : array-like, shape = (n_samples,)
+            Training vector, where n_samples is the number of samples.
+
+        y : array-like, shape = (n_samples,)
+            Target vector relative to x.
+
+        check_input : bool (default=False)
+            Whether to check input arrays.
+
+        Returns
+        -------
+        self : object
+            Fitted optimal binning.
+        """
+        return self._fit(x, y, check_input)
+
     def fit_transform(self, x, y, metric="mean_woe", metric_special=0,
-                      metric_missing=0, check_input=False):
+                      metric_missing=0, show_digits=2, check_input=False):
         """Fit the optimal binning according to the given training data, then
         transform it.
 
@@ -364,9 +389,11 @@ class MulticlassOptimalBinning(OptimalBinning):
 
         metric : str, optional (default="mean_woe")
             The metric used to transform the input vector. Supported metrics
-            are "mean_woe" to choose the mean of Weight of Evidence (WoE) and
+            are "mean_woe" to choose the mean of Weight of Evidence (WoE),
             "weighted_mean_woe" to choose weighted mean of WoE using the
-            number of records per class as weights.
+            number of records per class as weights, "indices" to assign the
+            corresponding indices of the bins and "bins" to assign the
+            corresponding bin interval.
 
         metric_special : float or str (default=0)
             The metric value to transform special codes in the input vector.
@@ -378,6 +405,10 @@ class MulticlassOptimalBinning(OptimalBinning):
             Supported metrics are "empirical" to use the empirical mean WoE
             or weighted mean WoE, and any numerical value.
 
+        show_digits : int, optional (default=2)
+            The number of significant digits of the bin column. Applies when
+            ``metric="bins"``.
+
         check_input : bool (default=False)
             Whether to check input arrays.
 
@@ -387,10 +418,11 @@ class MulticlassOptimalBinning(OptimalBinning):
             Transformed array.
         """
         return self.fit(x, y, check_input).transform(
-            x, metric, metric_special, metric_missing, check_input)
+            x, metric, metric_special, metric_missing, show_digits,
+            check_input)
 
     def transform(self, x, metric="mean_woe", metric_special=0,
-                  metric_missing=0, check_input=False):
+                  metric_missing=0, show_digits=2, check_input=False):
         """Transform given data to mean Weight of Evidence (WoE) or weighted
         mean WoE using bins from the fitted optimal binning.
 
@@ -401,9 +433,11 @@ class MulticlassOptimalBinning(OptimalBinning):
 
         metric : str, optional (default="mean_woe")
             The metric used to transform the input vector. Supported metrics
-            are "mean_woe" to choose the mean of Weight of Evidence (WoE) and
+            are "mean_woe" to choose the mean of Weight of Evidence (WoE),
             "weighted_mean_woe" to choose weighted mean of WoE using the
-            number of records per class as weights.
+            number of records per class as weights, "indices" to assign the
+            corresponding indices of the bins and "bins" to assign the
+            corresponding bin interval.
 
         metric_special : float or str (default=0)
             The metric value to transform special codes in the input vector.
@@ -415,6 +449,10 @@ class MulticlassOptimalBinning(OptimalBinning):
             Supported metrics are "empirical" to use the empirical mean WoE
             or weighted mean WoE, and any numerical value.
 
+        show_digits : int, optional (default=2)
+            The number of significant digits of the bin column. Applies when
+            ``metric="bins"``.
+
         check_input : bool (default=False)
             Whether to check input arrays.
 
@@ -428,31 +466,32 @@ class MulticlassOptimalBinning(OptimalBinning):
         return transform_multiclass_target(self._splits_optimal, x,
                                            self._n_event, self.special_codes,
                                            metric,  metric_special,
-                                           metric_missing, check_input)
+                                           metric_missing, show_digits,
+                                           check_input)
 
     def _fit(self, x, y, check_input):
         time_init = time.perf_counter()
 
         if self.verbose:
-            logging.info("Optimal binning started.")
-            logging.info("Options: check parameters.")
+            self._logger.info("Optimal binning started.")
+            self._logger.info("Options: check parameters.")
 
         _check_parameters(**self.get_params())
 
         # Pre-processing
         if self.verbose:
-            logging.info("Pre-processing started.")
+            self._logger.info("Pre-processing started.")
 
         self._n_samples = len(x)
 
         if self.verbose:
-            logging.info("Pre-processing: number of samples: {}"
-                         .format(self._n_samples))
+            self._logger.info("Pre-processing: number of samples: {}"
+                              .format(self._n_samples))
 
         time_preprocessing = time.perf_counter()
 
         [x_clean, y_clean, x_missing, y_missing, x_special, y_special,
-         _, _, _] = split_data(
+         _, _, _, _, _, _, _] = split_data(
             self.dtype, x, y, special_codes=self.special_codes,
             check_input=check_input, outlier_detector=self.outlier_detector,
             outlier_params=self.outlier_params)
@@ -469,26 +508,26 @@ class MulticlassOptimalBinning(OptimalBinning):
             n_missing = len(x_missing)
             n_special = len(x_special)
 
-            logging.info("Pre-processing: number of clean samples: {}"
-                         .format(n_clean))
+            self._logger.info("Pre-processing: number of clean samples: {}"
+                              .format(n_clean))
 
-            logging.info("Pre-processing: number of missing samples: {}"
-                         .format(n_missing))
+            self._logger.info("Pre-processing: number of missing samples: {}"
+                              .format(n_missing))
 
-            logging.info("Pre-processing: number of special samples: {}"
-                         .format(n_special))
+            self._logger.info("Pre-processing: number of special samples: {}"
+                              .format(n_special))
 
             if self.outlier_detector is not None:
                 n_outlier = self._n_samples-(n_clean + n_missing + n_special)
-                logging.info("Pre-processing: number of outlier samples: {}"
-                             .format(n_outlier))
+                self._logger.info("Pre-processing: number of outlier samples: "
+                                  "{}".format(n_outlier))
 
-            logging.info("Pre-processing terminated. Time: {:.4f}s"
-                         .format(self._time_preprocessing))
+            self._logger.info("Pre-processing terminated. Time: {:.4f}s"
+                              .format(self._time_preprocessing))
 
         # Pre-binning
         if self.verbose:
-            logging.info("Pre-binning started.")
+            self._logger.info("Pre-binning started.")
 
         time_prebinning = time.perf_counter()
 
@@ -496,13 +535,21 @@ class MulticlassOptimalBinning(OptimalBinning):
             n_splits = len(self.user_splits)
 
             if self.verbose:
-                logging.info("Pre-binning: user splits supplied: {}"
-                             .format(n_splits))
+                self._logger.info("Pre-binning: user splits supplied: {}"
+                                  .format(n_splits))
 
             user_splits = check_array(self.user_splits, ensure_2d=False,
                                       dtype=None, force_all_finite=True)
 
-            user_splits = np.unique(self.user_splits)
+            if len(set(user_splits)) != len(user_splits):
+                raise ValueError("User splits are not unique.")
+
+            sorted_idx = np.argsort(user_splits)
+            user_splits = user_splits[sorted_idx]
+
+            if self.user_splits_fixed is not None:
+                self.user_splits_fixed = np.asarray(
+                    self.user_splits_fixed)[sorted_idx]
 
             splits, n_nonevent, n_event = self._prebinning_refinement(
                 user_splits, x_clean, y_clean, y_missing, y_special, None)
@@ -515,21 +562,21 @@ class MulticlassOptimalBinning(OptimalBinning):
         self._time_prebinning = time.perf_counter() - time_prebinning
 
         if self.verbose:
-            logging.info("Pre-binning: number of prebins: {}"
-                         .format(self._n_prebins))
-            logging.info("Pre-binning: number of refinements: {}"
-                         .format(self._n_refinements))
+            self._logger.info("Pre-binning: number of prebins: {}"
+                              .format(self._n_prebins))
+            self._logger.info("Pre-binning: number of refinements: {}"
+                              .format(self._n_refinements))
 
-            logging.info("Pre-binning terminated. Time: {:.4f}s"
-                         .format(self._time_prebinning))
+            self._logger.info("Pre-binning terminated. Time: {:.4f}s"
+                              .format(self._time_prebinning))
 
         # Optimization
         self._fit_optimizer(splits, n_nonevent, n_event)
 
         # Post-processing
         if self.verbose:
-            logging.info("Post-processing started.")
-            logging.info("Post-processing: compute binning information.")
+            self._logger.info("Post-processing started.")
+            self._logger.info("Post-processing: compute binning information.")
 
         time_postprocessing = time.perf_counter()
 
@@ -549,24 +596,27 @@ class MulticlassOptimalBinning(OptimalBinning):
         self._time_postprocessing = time.perf_counter() - time_postprocessing
 
         if self.verbose:
-            logging.info("Post-processing terminated. Time: {:.4f}s"
-                         .format(self._time_postprocessing))
+            self._logger.info("Post-processing terminated. Time: {:.4f}s"
+                              .format(self._time_postprocessing))
 
         self._time_total = time.perf_counter() - time_init
 
         if self.verbose:
-            logging.info("Optimal binning terminated. Status: {}. "
-                         "Time: {:.4f}s".format(
-                            self._status, self._time_total))
+            self._logger.info("Optimal binning terminated. Status: {}. "
+                              "Time: {:.4f}s"
+                              .format(self._status, self._time_total))
 
         # Completed successfully
-        self._logger.close()
+        self._class_logger.close()
         self._is_fitted = True
 
         return self
 
     def _prebinning_refinement(self, splits_prebinning, x, y, y_missing,
-                               y_special, y_others=None):
+                               y_special, y_others=None, sw_clean=None,
+                               sw_missing=None, sw_special=None,
+                               sw_others=None):
+
         self._classes = np.unique(y)
         self._n_classes = len(self._classes)
 
@@ -598,7 +648,7 @@ class MulticlassOptimalBinning(OptimalBinning):
 
     def _fit_optimizer(self, splits, n_nonevent, n_event):
         if self.verbose:
-            logging.info("Optimizer started.")
+            self._logger.info("Optimizer started.")
 
         time_init = time.perf_counter()
 
@@ -608,10 +658,10 @@ class MulticlassOptimalBinning(OptimalBinning):
             self._solution = np.zeros(len(splits)).astype(np.bool)
 
             if self.verbose:
-                logging.warning("Optimizer: no bins after pre-binning.")
-                logging.warning("Optimizer: solver not run.")
+                self._logger.warning("Optimizer: no bins after pre-binning.")
+                self._logger.warning("Optimizer: solver not run.")
 
-                logging.info("Optimizer terminated. Time: 0s")
+                self._logger.info("Optimizer terminated. Time: 0s")
             return
 
         if self.min_bin_size is not None:
@@ -634,8 +684,8 @@ class MulticlassOptimalBinning(OptimalBinning):
                          for i in range(len(self._classes))]
 
             if self.verbose:
-                logging.info("Optimizer: classifier predicts {} monotonic "
-                             "trends.".format(monotonic))
+                self._logger.info("Optimizer: classifier predicts {} "
+                                  "monotonic trends.".format(monotonic))
         elif isinstance(self.monotonic_trend, list):
             if len(self.monotonic_trend) != self._n_classes:
                 raise ValueError("List of monotonic trends must be of size "
@@ -663,22 +713,22 @@ class MulticlassOptimalBinning(OptimalBinning):
                     monotonic.append(trend)
 
                     if self.verbose:
-                        logging.info("Optimizer: classifier predicts {} "
-                                     "monotonic trend.".format(trend))
+                        self._logger.info("Optimizer: classifier predicts {} "
+                                          "monotonic trend.".format(trend))
                 else:
                     monotonic.append(m_trend)
         elif self.monotonic_trend is None:
             monotonic = [None] * self._n_classes
 
             if self.verbose:
-                logging.info("Optimizer: monotonic trend not set.")
+                self._logger.info("Optimizer: monotonic trend not set.")
 
         if self.solver == "cp":
             optimizer = MulticlassBinningCP(monotonic, self.min_n_bins,
                                             self.max_n_bins, min_bin_size,
                                             max_bin_size, self.max_pvalue,
                                             self.max_pvalue_policy,
-                                            self._user_splits_fixed,
+                                            self.user_splits_fixed,
                                             self.time_limit)
         else:
             optimizer = MulticlassBinningMIP(monotonic, self.min_n_bins,
@@ -686,15 +736,15 @@ class MulticlassOptimalBinning(OptimalBinning):
                                              max_bin_size, self.max_pvalue,
                                              self.max_pvalue_policy,
                                              self.mip_solver,
-                                             self._user_splits_fixed,
+                                             self.user_splits_fixed,
                                              self.time_limit)
         if self.verbose:
-            logging.info("Optimizer: build model...")
+            self._logger.info("Optimizer: build model...")
 
         optimizer.build_model(n_nonevent, n_event, trend_changes)
 
         if self.verbose:
-            logging.info("Optimizer: solve...")
+            self._logger.info("Optimizer: solve...")
 
         status, solution = optimizer.solve()
 
@@ -707,8 +757,8 @@ class MulticlassOptimalBinning(OptimalBinning):
         self._time_solver = time.perf_counter() - time_init
 
         if self.verbose:
-            logging.info("Optimizer terminated. Time: {:.4f}s"
-                         .format(self._time_solver))
+            self._logger.info("Optimizer terminated. Time: {:.4f}s"
+                              .format(self._time_solver))
 
     def _compute_prebins(self, splits_prebinning, x, y):
         n_splits = len(splits_prebinning)
@@ -741,8 +791,8 @@ class MulticlassOptimalBinning(OptimalBinning):
                 [mask_remove[:-2], [mask_remove[-2] | mask_remove[-1]]])
 
             if self.user_splits_fixed is not None:
-                user_splits_fixed = np.asarray(self._user_splits_fixed)
-                user_splits = np.asarray(self._user_splits)
+                user_splits_fixed = np.asarray(self.user_splits_fixed)
+                user_splits = np.asarray(self.user_splits)
                 fixed_remove = user_splits_fixed & mask_splits
 
                 if any(fixed_remove):
@@ -752,14 +802,14 @@ class MulticlassOptimalBinning(OptimalBinning):
                                      .format(user_splits[fixed_remove]))
 
                 # Update boolean array of fixed user splits.
-                self._user_splits_fixed = user_splits_fixed[~mask_splits]
-                self._user_splits = user_splits[~mask_splits]
+                self.user_splits_fixed = user_splits_fixed[~mask_splits]
+                self.user_splits = user_splits[~mask_splits]
 
             splits = splits_prebinning[~mask_splits]
 
             if self.verbose:
-                logging.info("Pre-binning: number prebins removed: {}"
-                             .format(np.count_nonzero(mask_remove)))
+                self._logger.info("Pre-binning: number prebins removed: {}"
+                                  .format(np.count_nonzero(mask_remove)))
 
             [splits_prebinning, n_nonevent, n_event] = self._compute_prebins(
                 splits, x, y)
